@@ -1,6 +1,6 @@
-/* commonhelp.h: handle command line help common to omindex and scriptindex
+/* commonhelp.h: handle command line help common to omindex and scriptindex.
  *
- * Copyright (C) 2005,2006 Olly Betts
+ * Copyright (C) 2005,2006,2007 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -18,6 +18,11 @@
  * USA
  */
 
+#ifndef OMEGA_INCLUDED_COMMONHELP_H
+#define OMEGA_INCLUDED_COMMONHELP_H
+
 void print_package_info(const char *name);
 void print_stemmer_help(const char *spaces);
 void print_help_and_version_help(const char *spaces, char short_version = 'v');
+
+#endif // OMEGA_INCLUDED_COMMONHELP_H
