@@ -2,7 +2,7 @@
  *
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2001 Ananova Ltd
- * Copyright 2002,2006 Olly Betts
+ * Copyright 2002,2006,2007 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -22,17 +22,20 @@
 
 #include <config.h>
 
+#include "htmlparse.h"
+
 #include <xapian.h>
 
+#include "utf8convert.h"
+
 #include <algorithm>
-using std::find;
-using std::find_if;
+
+#include <ctype.h>
+#include <cstring>
 #include <stdio.h>
 #include <stdlib.h>
-#include <ctype.h>
 
-#include "htmlparse.h"
-#include "utf8convert.h"
+using namespace std;
 
 map<string, unsigned int> HtmlParser::named_ents;
 
