@@ -3,7 +3,7 @@
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2001,2005 James Aylett
  * Copyright 2001,2002 Ananova Ltd
- * Copyright 2002,2003,2004,2005,2006,2007 Olly Betts
+ * Copyright 2002,2003,2004,2005,2006,2007,2008 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -227,9 +227,23 @@ index_file(const string &url, const string &mimetype, time_t last_mod, off_t siz
 	md5_string(text, md5);
     } else if (mimetype == "text/plain") {
 	try {
-	    // Currently we assume that text files are UTF-8.
-	    // FIXME: What charset is the file?  Look for BOM?  Look at contents?
+	    // Currently we assume that text files are UTF-8 unless they have a
+	    // byte-order mark.
 	    dump = file_to_string(file);
+
+	    // Look for Byte-Order Mark (BOM).
+	    if (startswith(dump, "\xfe\xff") || startswith(dump, "\xff\xfe")) {
+		// UTF-16 in big-endian/little-endian order - we just convert
+		// it as "UTF-16" and let the conversion handle the BOM as that
+		// way we avoid the copying overhead of erasing 2 bytes from
+		// the start of dump.
+		convert_to_utf8(dump, "UTF-16");
+	    } else if (startswith(dump, "\xef\xbb\xbf")) {
+		// UTF-8 with stupid Windows not-the-byte-order mark.
+		dump.erase(0, 3);
+	    } else {
+		// FIXME: What charset is the file?  Look at contents?
+	    }
 	} catch (ReadError) {
 	    cout << "can't read \"" << file << "\" - skipping\n";
 	    return;
@@ -353,8 +367,10 @@ index_file(const string &url, const string &mimetype, time_t last_mod, off_t siz
 	// FIXME: Implement support for metadata.
 	try {
 	    XmlParser xmlparser;
-	    xmlparser.parse_html(file_to_string(file));
+	    string text = file_to_string(file);
+	    xmlparser.parse_html(text);
 	    dump = xmlparser.dump;
+	    md5_string(text, md5);
 	} catch (ReadError) {
 	    cout << "can't read \"" << file << "\" - skipping\n";
 	    return;
@@ -372,7 +388,7 @@ index_file(const string &url, const string &mimetype, time_t last_mod, off_t siz
 	}
     } else if (mimetype == "text/rtf") {
 	// The --text option unhelpfully converts all non-ASCII characters to
-	// "?" so we use --html instead, which write HTML entities.
+	// "?" so we use --html instead, which produces HTML entities.
 	string cmd = "unrtf --nopict --html 2>/dev/null " + shell_protect(file);
 	MyHtmlParser p;
 	try {
@@ -404,7 +420,7 @@ index_file(const string &url, const string &mimetype, time_t last_mod, off_t siz
 	    return;
 	}
     } else if (mimetype == "application/x-dvi") {
-	// FIXME: -e2 means "UTF-8", but that results in "fi", "ff", "ffi", etc
+	// FIXME: -e0 means "UTF-8", but that results in "fi", "ff", "ffi", etc
 	// appearing as single ligatures.  For European languages, it's
 	// actually better to use -e2 (ISO-8859-1) and then convert, so let's
 	// do that for now until we handle Unicode "compatibility
@@ -417,6 +433,18 @@ index_file(const string &url, const string &mimetype, time_t last_mod, off_t siz
 	    cout << "\"" << cmd << "\" failed - skipping\n";
 	    return;
 	}
+    } else if (mimetype == "image/vnd.djvu") {
+	// Output is UTF-8 according to "man djvutxt".  Generally this seems to
+	// be true, though some examples from djvu.org generate isolated byte
+	// 0x95 in a context which suggests it might be intended to be a bullet
+	// (as it is in CP1250).
+	string cmd = "djvutxt " + shell_protect(file);
+	try {
+	    dump = stdout_to_string(cmd);
+	} catch (ReadError) {
+	    cout << "\"" << cmd << "\" failed - skipping\n";
+	    return;
+	}
     } else {
 	// Don't know how to index this type.
 	cout << "unknown MIME type - skipping\n";
@@ -535,64 +563,84 @@ index_directory(size_t depth_limit, const string &dir,
     DirectoryIterator d(follow_symlinks);
     try {
 	d.start(path);
-    } catch (const std::string & error) {
-	cout << error << " - skipping" << endl;
-	return;
-    }
-    while (d.next()) try {
-	string url = dir;
-	if (!url.empty() && url[url.size() - 1] != '/') url += '/';
-	url += d.leafname();
-	string file = root + indexroot + url;
-	switch (d.get_type()) {
-	    case DirectoryIterator::DIRECTORY:
-		if (depth_limit == 1) continue;
-		try {
-		    size_t new_limit = depth_limit;
-		    if (new_limit) --new_limit;
-		    index_directory(new_limit, url, mime_map);
-		} catch (...) {
-		    cout << "Caught unknown exception in index_directory, rethrowing" << endl;
-		    throw;
-		}
-		continue;
-	    case DirectoryIterator::REGULAR_FILE: {
-		string ext;
-		string::size_type dot = url.find_last_of('.');
-		if (dot != string::npos) ext = url.substr(dot + 1);
-
-		map<string,string>::iterator mt = mime_map.find(ext);
-		if (mt != mime_map.end()) {
-		    // Only check the file size if we recognise the extension
-		    // to avoid a call to stat()/lstat() for files we can't
-		    // handle when readdir() tells us the file type.
-		    off_t size = d.get_size();
-		    if (size == 0) {
-			cout << "Skipping empty file: \"" << file << "\"" << endl;
-			continue;
-		    }
-
-		    // It's in our MIME map so we know how to index it.
-		    const string & mimetype = mt->second;
+	while (d.next()) try {
+	    string url = dir;
+	    if (!url.empty() && url[url.size() - 1] != '/') url += '/';
+	    url += d.leafname();
+	    string file = root + indexroot + url;
+	    switch (d.get_type()) {
+		case DirectoryIterator::DIRECTORY:
+		    if (depth_limit == 1) continue;
 		    try {
-			time_t mtime = d.get_mtime();
-			index_file(indexroot + url, mimetype, mtime, size);
-		    } catch (NoSuchFilter) {
-			// FIXME: we ought to ignore by mime-type not extension.
-			cout << "Filter for \"" << mimetype << "\" not installed - ignoring extension \"" << ext << "\"" << endl;
-			mime_map.erase(mt);
+			size_t new_limit = depth_limit;
+			if (new_limit) --new_limit;
+			index_directory(new_limit, url, mime_map);
+		    } catch (...) {
+			cout << "Caught unknown exception in index_directory, rethrowing" << endl;
+			throw;
 		    }
-		} else {
-		    cout << "Unknown extension: \"" << file << "\" - skipping" << endl;
+		    continue;
+		case DirectoryIterator::REGULAR_FILE: {
+		    string ext;
+		    string::size_type dot = url.find_last_of('.');
+		    if (dot != string::npos) ext = url.substr(dot + 1);
+
+		    map<string,string>::iterator mt = mime_map.find(ext);
+		    if (mt == mime_map.end()) {
+			// If the extension isn't found, see if the lower-cased
+			// version (if different) is found.
+			bool changed = false;
+			string::iterator i;
+			for (i = ext.begin(); i != ext.end(); ++i) {
+			    if (*i >= 'A' && *i <= 'Z') {
+				*i = tolower(*i);
+				changed = true;
+			    }
+			}
+			if (changed) mt = mime_map.find(ext);
+		    }
+		    if (mt != mime_map.end()) {
+			// Only check the file size if we recognise the
+			// extension to avoid a call to stat()/lstat() for
+			// files we can't handle when readdir() tells us the
+			// file type.
+			off_t size = d.get_size();
+			if (size == 0) {
+			    cout << "Skipping empty file: \"" << file << "\""
+				 << endl;
+			    continue;
+			}
+
+			// It's in our MIME map so we know how to index it.
+			const string & mimetype = mt->second;
+			try {
+			    time_t mtime = d.get_mtime();
+			    index_file(indexroot + url, mimetype, mtime, size);
+			} catch (NoSuchFilter) {
+			    // FIXME: we ought to ignore by mime-type not
+			    // extension.
+			    cout << "Filter for \"" << mimetype
+				 << "\" not installed - ignoring extension \""
+				 << ext << "\"" << endl;
+			    mime_map.erase(mt);
+			}
+		    } else {
+			cout << "Unknown extension: \"" << file
+			     << "\" - skipping" << endl;
+		    }
+		    continue;
 		}
-		continue;
+		default:
+		    cout << "Not a regular file \"" << file
+			 << "\" - skipping" << endl;
 	    }
-	    default:
-		cout << "Not a regular file \"" << file << "\" - skipping\n";
+	} catch (const std::string & error) {
+	    cout << error << " - skipping" << endl;
+	    continue;
 	}
     } catch (const std::string & error) {
-	cout << error << " - skipping" << endl;
-	continue;
+	cout << error << " - skipping directory" << endl;
+	return;
     }
 }
 
@@ -688,10 +736,13 @@ main(int argc, char **argv)
     mime_map["pl"] = "text/x-perl";
     mime_map["pm"] = "text/x-perl";
     mime_map["pod"] = "text/x-perl";
-    // Other formats:
+    // TeX DVI:
     mime_map["dvi"] = "application/x-dvi";
+    // DjVu:
+    mime_map["djv"] = "image/vnd.djvu";
+    mime_map["djvu"] = "image/vnd.djvu";
 
-    while ((getopt_ret = gnu_getopt_long(argc, argv, "hvd:D:U:M:lp", longopts, NULL))!=EOF) {
+    while ((getopt_ret = gnu_getopt_long(argc, argv, "hvd:D:U:M:lpf", longopts, NULL)) != -1) {
 	switch (getopt_ret) {
 	case 'h': {
 	    cout << PROG_NAME" - "PROG_DESC"\n\n"
