@@ -4,7 +4,8 @@
  * Copyright 2001 James Aylett
  * Copyright 2001,2002 Ananova Ltd
  * Copyright 2002 Intercede 1749 Ltd
- * Copyright 2002,2003,2004,2005,2006,2007 Olly Betts
+ * Copyright 2002,2003,2004,2005,2006,2007,2008 Olly Betts
+ * Copyright 2008 Thomas Viehmann
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -653,11 +654,17 @@ html_highlight(const string &s, const string &list,
 		res += bra;
 	    } else {
 		static const char * colours[] = {
-		    "ffff66", "66ffff", "ff66ff", "6666ff", "ff6666",
-		    "66ff66", "ffaa33", "33ffaa", "aa33ff", "33aaff"
+		    "ffff66", "99ff99", "99ffff", "ff66ff", "ff9999",
+		    "990000", "009900", "996600", "006699", "990099"
 		};
-		res += "<b style=\"color:black;background-color:#";
-		res += colours[match % (sizeof(colours) / sizeof(colours[0]))];
+		size_t idx = match % (sizeof(colours) / sizeof(colours[0]));
+		const char * bg = colours[idx];
+		if (strchr(bg, 'f')) {
+		    res += "<b style=\"color:black;background-color:#";
+		} else {
+		    res += "<b style=\"color:white;background-color:#";
+		}
+		res += bg;
 		res += "\">";
 	    }
 	    word = string(first.raw(), j.raw() - first.raw());
@@ -715,6 +722,7 @@ static string print_caption(const string &fmt, const vector<string> &param);
 enum tagval {
 CMD_,
 CMD_add,
+CMD_addfilter,
 CMD_allterms,
 CMD_and,
 CMD_cgi,
@@ -756,6 +764,7 @@ CMD_length,
 CMD_list,
 CMD_log,
 CMD_lookup,
+CMD_lower,
 CMD_lt,
 CMD_map,
 CMD_max,
@@ -801,6 +810,7 @@ CMD_transform,
 CMD_uniq,
 CMD_unpack,
 CMD_unstem,
+CMD_upper,
 CMD_url,
 CMD_value,
 CMD_version,
@@ -824,11 +834,12 @@ struct func_desc {
 #define M 'M'
 #define Q 'Q'
 // NB when adding a new command which ensures M or Q, update the list in
-// docs/omegascript.txt
+// docs/omegascript.rst
 static struct func_desc func_tab[] = {
 //name minargs maxargs evalargs ensure
 {"",{CMD_,	   N, N, 0, 0}},// commented out code
 T(add,		   0, N, N, 0), // add a list of numbers
+T(addfilter,	   1, 1, N, 0), // add filter term
 T(allterms,	   0, 1, N, 0), // list of all terms matching document
 T(and,		   1, N, 0, 0), // logical shortcutting and of a list of values
 T(cgi,		   1, 1, N, 0), // return cgi parameter value
@@ -872,6 +883,7 @@ T(length,	   1, 1, N, 0), // length of list
 T(list,		   2, 5, N, 0), // pretty print list
 T(log,		   1, 2, 1, 0), // create a log entry
 T(lookup,	   2, 2, N, 0), // lookup in named cdb file
+T(lower,	   1, 1, N, 0), // convert string to lower case
 T(lt,		   2, 2, N, 0), // test <
 T(map,		   1, 2, 1, 0), // map a list into another list
 T(max,		   1, N, N, 0), // maximum of a list of values
@@ -920,6 +932,7 @@ T(uniq,		   1, 1, N, 0), // removed duplicates from a sorted list
 T(unpack,	   1, 1, N, 0), // convert 4 byte big endian binary string to a number
 T(unstem,	   1, 1, N, Q), // return list of probabilistic terms from
 				// the query which stemmed to this term
+T(upper,	   1, 1, N, 0), // convert string to upper case
 T(url,		   1, 1, N, 0), // url encode argument
 T(value,	   1, 2, N, 0), // return document value
 T(version,	   0, 0, N, 0), // omega version string
@@ -1057,6 +1070,9 @@ eval(const string &fmt, const vector<string> &param)
 		value = int_to_string(total);
 		break;
 	    }
+	    case CMD_addfilter:
+		add_bterm(args[0]);
+		break;
 	    case CMD_allterms: {
 		// list of all terms indexing document
 		int id = q0;
@@ -1190,8 +1206,8 @@ eval(const string &fmt, const vector<string> &param)
 			    format = "%d.%cG";
 			}
 		    }
-		    intpart = size / 1024;
-		    fraction = size % 1024;
+		    intpart = unsigned(size) / 1024;
+		    fraction = unsigned(size) % 1024;
 		}
 		if (format) {
 		    char buf[200];
@@ -1247,7 +1263,9 @@ eval(const string &fmt, const vector<string> &param)
 	    case CMD_freq:
 		try {
 		    value = int_to_string(mset.get_termfreq(args[0]));
-		} catch (...) {
+		} catch (const Xapian::InvalidOperationError&) {
+		    // An mset will raise this error if it's empty and not associated with
+		    // a search.
 		    value = int_to_string(db.get_termfreq(args[0]));
 		}
 		break;
@@ -1456,6 +1474,9 @@ eval(const string &fmt, const vector<string> &param)
 		close(fd); // FIXME: cache fds?
 		break;
 	    }
+	    case CMD_lower:
+		value = Xapian::Unicode::tolower(args[0]);
+		break;
             case CMD_lt:
 		if (string_to_int(args[0]) < string_to_int(args[1]))
 		    value = "true";
@@ -1889,6 +1910,9 @@ eval(const string &fmt, const vector<string> &param)
 		}
 		break;
 	    }
+	    case CMD_upper:
+		value = Xapian::Unicode::toupper(args[0]);
+		break;
 	    case CMD_url:
 	        value = percent_encode(args[0]);
 		break;
